#!/usr/bin/env python3
# v0.33  27 December 2017, author: Brian Fiedler
# for plotting pkl files output from gocfd4.py
# usage example: arf4plot.py g01M025/torn022500.pkl 
# special plot of buoyancy and grid: # arf4plot.py g01M025/torn022500.pkl X 
# this program is not documented well
from __future__ import print_function
import sys 
try:
    import cPickle as pickle #python 2
except:
    import pickle as pickle #python 3 
import matplotlib; matplotlib.use('Agg') # so can be used with no X11 connection
from matplotlib.pyplot import *
from matplotlib import rc
import matplotlib.lines as mpllines
import glob

buoygridplot=False 
fieldstoplot='u v w pp'.split()
# IMPORTANT HACK:
if len(sys.argv)>2:
    buoygridplot=True #set True to make special plot of buoyancy and grid
    print("making plot of buoyancy and grid")
    fieldstoplot='b v w pp'.split() # to see the buoyancy instead of u

rc("xtick", labelsize=18)
rc("ytick", labelsize=18)

makeplots=True # obviously?
tcol='k'
loc=-2.05 # highest contour
hic=2.05 #lowest contour
dc=.1 #contour interval

try:
    infilename=sys.argv[1]
    if infilename[-4:]!='.pkl': # treat as dir name, get latest pkl file
        infilename=sorted(glob.glob(infilename+'/*.pkl'))[-1]
    print("attempting to open: ", infilename)
    infile=open(infilename,'rb')
except:
    sys.exit("quitting, you need to specify valid file path")


s=infilename.split('/')
outfilename=s[-2]+'_'+s[-1].split('.')[0][-6:]+'.eps'
parms,fields=pickle.load(infile)
#print(parms)
#print(fields.keys())
xa=fields['x']
y=fields['y']
if y.shape[0] < 47:
    imax=24
elif y.shape[0] <100:
    imax=47 # restrict to central domain
else:
    imax=93 # restrict to central domain
print(imax, xa[0,imax-2], xa[0,imax-1], xa[0,imax] )
xlim=xa[0,imax-1]
ylim=y[-1,0]
x=fields['x'][:,:imax]
y=fields['y'][:,:imax]
pp = fields['pp']
u = fields['u']
v = fields['v']
w = fields['w']
b = fields['b']
z=fields['y']
if 'q' in fields:
    q = fields['q']
else:
    q=0*w
print("using q",q.max(),q.min())
hd = (pp +.5*u*u + .5*v*v +.5*w*w) - q
hd = hd- hd[0,imax-1]
fields['hd']=hd
fieldstoplot.append('hd')
###
#q = fields['q']
#fieldstoplot.append('q')

for k in fieldstoplot:
    f=fields[k][:,:imax]
    fmax=f.max()
    fmin=f.min()
    print(k,fmin,fmax)
############################### 
def setsubplot(n):
    global sa
    sa=axes([(dw+n*(dpw+din))/guiasp,dw,dpw/guiasp,pt])
#    sa.set_xticks([0,xlim])
#    sa.set_yticks([0,ylim])
    sa.set_xticks([]) # for NO ticks
    sa.set_yticks([]) # for NO ticks
    lines =sa.get_xticklines()
    for line in lines:
        line.set_marker(mpllines.TICKDOWN)
        line.set_color(tcol)
    lines =sa.get_yticklines()
    for line in lines:
        line.set_marker(mpllines.TICKLEFT)
        line.set_color(tcol)
        line.set_linewidth(10)
    labs=sa.get_xticklabels()
    for lab in labs:
        lab.set_color(tcol)
    labs=sa.get_yticklabels()
    for lab in labs:
        lab.set_color(tcol)
    for axis in ['top','bottom','left','right']:
        sa.spines[axis].set_linewidth(0.1)
    sa.spines['right'].set_linewidth(0.)
    sa.set_frame_on(True)
def texttop(f,lab,theaxes):
    fmin=f.min()
    fmax=f.max()
    fex=max(abs(fmin),abs(fmax))
    formt=r'%5.2f'
    if fex<.1: formt=r'%7.4f'
    if fex<.001: formt=r'%8.1e'
#    leftstring= formt+' <' 
    leftstring= formt+''
    leftstring= leftstring % (fmin)
#    rightstring= '< '+formt % (fmax)
    rightstring= ''+formt % (fmax)
    topstring= r'$%s$' % (lab)
    theaxes.text(.0*xlim,1.02*ylim  ,leftstring ,fontsize=36,
            verticalalignment='bottom',horizontalalignment='left',color=tcol)
    theaxes.text(xlim,1.02*ylim  ,rightstring ,fontsize=36,
            verticalalignment='bottom',horizontalalignment='right',color=tcol)
##    theaxes.text(.5*xlim,1.01*ylim  ,topstring ,fontsize=36,
##            verticalalignment='bottom',horizontalalignment='center',color=tcol)
###############
mylevs=[]
arfc=[]
fillcolors=[]
cvalue=loc
while cvalue < hic+.1*dc and len(mylevs)<100:
    mylevs.append(cvalue)
    if cvalue<0.:
        arfc.append('g')
        red = 1. -  abs(cvalue)/hic 
        blue = 1.
        green = 1.
        if cvalue+dc<0.:
            fillcolors.append( (red,green,blue) )
        else: 
            fillcolors.append( (1.,1.,1.) )
    else:
        arfc.append('r')
        red= 1.
        blue = 1.
        green = 1 - abs(cvalue+.05)/(hic+.05) 
        fillcolors.append( (red,green,blue) )
    cvalue+=dc
#print(mylevs,arfc,fillcolors)
def contourit(f,lab):
#    nothing=contour(x,y,f,mylevs,colors=arfc,linewidths=[.1],zorder=3)
    nothing=contour(x,y,f,mylevs,colors=arfc,linewidths=[.5],zorder=3)
    nothing=contourf(x,y,f,mylevs,colors=fillcolors,zorder=1)
#    texttop(f,lab,sa)
###########################################
def drawgrid(theaxes):
    im,jm= x.shape
    segs=[]
    regs=[]
    grcol='b' # [0,0,0]
    lskip=4
    for j in np.arange(0,jm,lskip):
        xl=x[:,j]
        yl=y[:,j]
        myline= Line2D(xl,yl,color=grcol,linewidth=.0001)
        myline.set_aa(False)
        theaxes.add_line(myline)
    for i in np.arange(0,im,lskip):
        xl=x[i,:]
        yl=y[i,:]
        myline= Line2D(xl,yl,color=grcol,linewidth=.0001)
        myline.set_aa(False)
        theaxes.add_line(myline)



if makeplots:
    guiheight=8.
#    pt=.7
    pt=.9 #tight
    plotasp=1. #means isotropic distances 
    dpw=pt*xlim/ylim
    din=.07
#    pw=plotasp*(dpw+din)*len(fieldstoplot)
    pw=plotasp*(dpw+din)*len(fieldstoplot)-plotasp*din #tight
#    dw=.1
    dw=.02 #tight
    cw=0.
    addcolorlegend=False
    if addcolorlegend: cw=.1
    guiwidth=(2*dw+pw+2*cw)*guiheight
    guiasp=guiwidth/guiheight
    figure(figsize=(guiwidth,guiheight),facecolor='w')
    nframes = len(fieldstoplot)
    for nframe in range( 0 , nframes ):
        setsubplot(nframe)
#        thelab=fieldstoplot[nframe]
        k=fieldstoplot[nframe]
        print("contouring",k)
        f=fields[k][:,:imax]
        if buoygridplot and nframe==1:
            drawgrid(sa)
            contourit(.0001*f,k)
            outfilename='buoy_grid.eps'
            break
        else:
            contourit(f,k)
            texttop(f,"",sa)

    print("saving figure in:",outfilename)
    savefig(outfilename, dpi=72, facecolor='w', edgecolor='w', orientation='portrait')
