#!/usr/bin/env python3
# v1.1, 1 August 2017, author: Brian Fiedler
# MIT license. 
# tested recently only for axisymmetric vortices 
# normally, this  is run from the tornado.py script, such as:
# tornado.py .01 .025
# .01 is gamma, the semi-slip coefficient
# .025 is Omega, the rotation rate of the chamber

from __future__ import print_function #if needed for python 2
import numpy as np
import sys
import npcfd3 as npcfd
try:
    import cPickle as pickle # python 2
except:
    print("assuming python 3")
    import pickle as pickle # python 3


import argparse
parser = argparse.ArgumentParser()
parser.add_argument("-d", dest="dblgrid",action="store_true")
parser.add_argument("-q", dest="doq",action="store_true")
parser.add_argument("--a5", dest="advect5",action="store_true")
parser.add_argument("--deltadump", dest="deltadump",default=1)
parser.add_argument("--pit", dest="pit",type=int,default=40)
parser.add_argument("infilename")
parser.add_argument("tstop",type=float)
opt = parser.parse_args()

axisym = True # I have not tested Cartesian in a while, leave this True

try:
    infilename=opt.infilename
    infile=open(opt.infilename,'rb')
except:
    sys.exit(' need valid infilename to start')

try:
    tstop = opt.tstop
except:
    sys.exit(' need tstop')

outstem=infilename
outstem=outstem.replace('.pkl','')
outstem=outstem[:-6]
print(outstem)
parms,fields=pickle.load(infile)
print(parms)
print(fields.keys())
parms['pit']=opt.pit
parms['advect5']=opt.advect5


###########
x=fields['x']
y=fields['y']
grid=npcfd.makegridcoef(x,y)
b=fields['b']
u=fields['u']
v=fields['v']
w=fields['w']
p=fields['p']
dco=fields['dco']
if 'q' in fields: # this is for the calculation of work done by buoyancy
    q = fields['q']
elif opt.doq:
    q = 0.*y.copy()
    fields['q'] = q
    print("initialized q with 0")

def doubleU(g,jmdbl,imdbl): # for doubling resolution on velocity grid
    gg = np.zeros((jmdbl,imdbl))
    gg[::2,::2] = g[:,:].copy()
    gg[1:-1:2,::2] = (gg[:-2:2,::2]+gg[2::2,::2])*.5 
    gg[::2,1::2] = (gg[::2,:-2:2]+gg[::2,2::2])*.5 
    gg[1::2,1::2] = .25*( g[:-1,:-1] + g[:-1,1:] + g[1:,:-1] + g[1:,1:])
    return gg

if opt.dblgrid: # double grid resolution of all fields
    jm,im = b.shape
    jm2 = (jm-1)*2 + 1
    im2 = (im-1)*2 + 1
    x,y = npcfd.makexy(im2,jm2,parms['width'],parms['height'],parms['stx'],parms['sty'])
    grid = npcfd.makegridcoef(x,y)
    dco = 0.*y + parms['dcoef'] 
    dcot = 2*parms['dcoeftop']*(y-.5)
    dco = np.where(dcot>dco,dcot,dco)
    b = doubleU(b,jm2,im2)
    u = doubleU(u,jm2,im2)
    v = doubleU(v,jm2,im2)
    w = doubleU(w,jm2,im2)
    pp = doubleU(fields['pp'],jm2,im2)
    p = (pp[:-1,:-1] + pp[:-1,1:] + pp[1:,:-1] + pp[1:,1:])*.25  
    fields['x'] = x
    fields['y'] = y
    fields['b'] = b
    fields['u'] = u
    fields['w'] = w
    fields['v'] = v
    fields['p'] = p
    fields['pp'] = pp
    fields['dco'] = dco
    if 'q' in fields:
        q = doubleU(q,jm2,im2)
        fields['q'] = q
    print("double resolution from",jm,im,"to",jm2,im2)

nstepmax=10000000
time=parms['time']
dtmax=.01
dtmin=1.e-5
cfl=.15
difflim=.1
alpha=.5
divrid=.1

cori=parms['cori']
gamma = parms['gamma']

abscheme=3
if parms['noslip']==1:
    print('using no-slip')
    noslip=[True,True,True,False] # for no-slip lower boundary
elif parms['noslip']==0:
    print('using semi-slip')
    noslip=[False,True,True,False] # for free-slip and semi-slip lower boundary
else:
    sys.exit('noslip incorrect')

if opt.advect5:
    advect=npcfd.advect5
else:
    advect=npcfd.advect3
 
sh0,sh1=x.shape
dtdta=[None]*3
dudta=[None]*3
dwdta=[None]*3
dvdta=[None]*3
if 'q' in fields: dqdta=[None]*3
dcoefx=.5*(dco[:,1:]+dco[:,:-1])
dcoefy=.5*(dco[1:,:]+dco[:-1,:])
nstep=0
jab=[0,1,2]
rdxa=grid['rdxa']
rdya=grid['rdya']
diffrate=dco*(rdxa**2+rdya**2) # for calculation time-step limitation 
dtdiff=dtmax # for case of dco=0 everywhere
if np.max(dco)>0.: dtdiff=difflim/np.max(diffrate)
dtadv=dtmax
if 'dt' in parms:
    dt=parms['dt']
else:
    dt=dtmax
dt1=dt
timedump=time+opt.deltadump
gameover=False
print("start loop",time,tstop,dt,dtmin,nstep)
dt = max(dt,dtmin*1.01)
while time< tstop+dt/2 and nstep<nstepmax and dt>dtmin and not gameover:
    dt2=dt1
    dt1=dt
    advrate=np.max(np.max(rdxa*u)+np.max(rdya*v))
    if advrate>0.: dtadv=cfl/advrate
    dt=min(dtmax,dtdiff,dtadv) # constrain time-step to be small
    if time+dt>tstop:
        dt=tstop-time
        gameover=True
    j=jab[0]
    dudt=advect(u,u,w,grid)
    dwdt=advect(w,u,w,grid)
#    dtdt=advect(t,u,w,grid)
    dvdt=advect(v,u,w,grid)
    if axisym==True: 
#        dtdt+=npcfd.diffuse_axi_w(t,dcoefx,dcoefy,grid)
        dwdt+=npcfd.diffuse_axi_w(w,dcoefx,dcoefy,grid)
        dudt+=npcfd.diffuse_axi_v(u,dcoefx,dcoefy,grid,semic=gamma)
        dvdt+=npcfd.diffuse_axi_v(v,dcoefx,dcoefy,grid,semic=gamma)
    else: # for Cartesian tests
#        dtdt+=npcfd.diffuse_cart(t,dcoefx,dcoefy,grid)
        dudt+=npcfd.diffuse_cart(u,dcoefx,dcoefy,grid)
        dwdt+=npcfd.diffuse_cart(w,dcoefx,dcoefy,grid)
        dvdt+=npcfd.diffuse_cart(v,dcoefx,dcoefy,grid)
    dvdt+=-cori*u 
    dvdt[:,1:]+=-v[:,1:]*u[:,1:]/x[:,1:]
    dudt[:,1:]+=v[:,1:]*v[:,1:]/x[:,1:]+cori*v[:,1:]
    dwdt[1:-1,:]+=b[1:-1,:]
    if 'q' in fields: #q is the work done by buoyancy
        dqdt=advect(q,u,w,grid) 
        dqdt[1:-1,:] += w[1:-1,:]*b[1:-1,:]
        dqdt += npcfd.diffuse_axi_w(q,dcoefx,dcoefy,grid) # smoother, not physical
    dwdt[0,:]=0.
    dwdt[-1,:]=0.
    dudt[:,-1]=0.
    dudt[:,0]=0.
    npcfd.velbca(dvdt,noslip)
    npcfd.velbc(dudt,dwdt,noslip)
    divergence=npcfd.div2(u,w,grid,axisym=axisym)
# adjust pressure to maintain non-divergence
    dudt,dwdt,p= npcfd.incomp3(dudt,dwdt,p,
            divrid*divergence/dt,grid,axisym=axisym,
            pit=opt.pit,alpha=alpha,noslip=noslip)

# ready for Adams-Bashforth update
    dudta = [dudt.copy()]+dudta[:-1]
    dvdta = [dvdt.copy()]+dvdta[:-1]
    dwdta = [dwdt.copy()]+dwdta[:-1]
    ab = npcfd.abcoef(dt1/dt,(dt2+dt1)/dt)
    abnow = min(abscheme-1,nstep)
    u += dt*npcfd.abstep(dudta,ab[abnow])
    w += dt*npcfd.abstep(dwdta,ab[abnow])
    v += dt*npcfd.abstep(dvdta,ab[abnow])
    if 'q' in fields:
        dqdta = [dqdt.copy()]+dqdta[:-1]
        q += dt*npcfd.abstep(dqdta,ab[abnow])
        q = np.where(x<0.6, q, 0.*x)
        fields['q'] = q # needed for q only, a new q was created in previous line
    time += dt
    fs="{0:8d} {1:10.3e} {2:10.3e}  {3:10.3e} {4:10.3e} {5:10.3e} {6:10.3e} {7:10.3e}"
    ffs=fs.format(nstep,dt,time,np.max(v),np.max(w),np.min(p), 
        np.max(divergence),np.min(divergence) )
    if (nstep%10==0): print(ffs)
    jab = jab[-1:]+jab[:-1] #rotate time tendencies
    nstep += 1
    if time+.00001*dt>timedump or gameover: # dump fields into file
        fields['pp']=npcfd.p2pu(p)
        parms['time']=time
        parms['cori']=cori
        parms['dt']=dt
        itime="%06d" % (int(100*time+.001*dt))
        outfilename=outstem+itime+'.pkl'
        ouf=open(outfilename,'wb')
        pickle.dump([parms,fields],ouf,-1)
        print('dumped: ',outfilename,'   time=',time,itime)
        timedump += opt.deltadump
        ouf.close()
