#!/usr/bin/env python3
# v1.1, 1 August 2017
# author: Brian Fiedler
from __future__ import print_function # if needed for python 2
import npcfd3 as nc
import numpy as np
import sys, os
try:
    import cPickle as pickle # python 2
except:
    print("assuming python3")
    import pickle as pickle # python 3
#from optparse import OptionParser
import argparse
parser = argparse.ArgumentParser()
#usage="usage: %prog dirname [options]"
#parser = OptionParser(usage=usage, version="%prog 0.1")
parser.add_argument("--width", dest="width",type=float,help="chamber radius", default=2.)
parser.add_argument("--height", dest="height",type=float,help="chamber height", default=1.)
parser.add_argument("--stx", dest="stx",type=float,help="stretch in r", default=.15)
parser.add_argument("--sty", dest="sty",type=float,help="stretch in z", default=.3)
parser.add_argument("--dcoef", dest="dcoef",type=float,help="viscosity", default=1.e-4)
parser.add_argument("--dcoeftop", dest="dcoeftop",type=float,help="viscosity at top", default=1.e-3)
parser.add_argument("--Omega", dest="Omega",type=float,help="Rotation rate", default=.025)
parser.add_argument("--gamma", dest="gamma",type=float,help="semi-slip parameter", default=.01)
parser.add_argument("--xpts", dest="xpts",type=int,help="number of points in r", default=46)
parser.add_argument("--ypts", dest="ypts",type=int,help="number of points in z", default=46)
#parser.add_argument("--dirname", dest="dirname",type=str,help="directory for pkl file",default="")
parser.add_argument("dirname")
#parser.add_option("-d", dest="doubleGrid",action="store_true",help="double resolution",default=False)
#parser.add_option("-c", dest="create",action="store_true",help="initialize pkl file",default=False)
#(opt,args)=parser.parse_args()
opt =parser.parse_args()
print(type(opt))
print(opt)
#print(args)
#width=2.
#height=1.
#stx=.15
#sty=.3
#dcoef=1.e-4 
#dcoeftop=1.e-3
x,y=nc.makexy(opt.xpts, opt.ypts, opt.width, opt.height, opt.stx, opt.sty)
#x,y=nc.makexy(91,91,width,height,stx,sty)
#x,y=nc.makexy(181,181,width,height,stx,sty)
print(dir(opt))
parms = vars(opt) 
print(type(parms),parms)
#parms={}
parms['time']=0.
cori=2.*opt.Omega
parms['cori']=cori
parms['gamma']=opt.gamma
if opt.gamma>9:
    parms['noslip']=1 # no-slip
else:
    parms['noslip']=0 # free-slip


#configure your start file
#swirl=int(sys.argv[1])
#boundc=int(sys.argv[2])
#semic=0.
#if (len(sys.argv)>3):
#    semic=float(sys.argv[3])
#parms['semic'] = semic


#bubsize=10.
#b=1.2636*np.exp(-10.*x**2 -20.*(y-.5)**2) #ancient way
rb=np.sqrt(x**2+(y-.5)**2) # for Hindawi publication  
b=np.where(rb<0.5,.5*(1.+np.cos(2*np.pi*rb)),0.) #for Hindawi publication
u=0.*y
w=0.*y
v=0.*y
p=0.*y[1:,1:]
pp=0.*y
dco=0.*y + opt.dcoef 
dcot=2*opt.dcoeftop*(y-.5)
dco=np.where(dcot>dco,dcot,dco)

fields={}
fields['x']=x
fields['y']=y
fields['b']=b
fields['u']=u
fields['w']=w
fields['v']=v
fields['p']=p
fields['pp']=pp
fields['dco']=dco
if len(opt.dirname)>0:
    if not os.path.isdir(opt.dirname):
        print("making new directory:",opt.dirname)
        os.mkdir(opt.dirname)
    outpklname = opt.dirname+"/"+"torn000000.pkl"
else:
    outpklname = "torn000000.pkl"
ouf=open(outpklname,'wb')
pickle.dump([parms,fields],ouf,-1)
ouf.close()
# test the pickle file
print('wrote ',outpklname )

inf=open(outpklname,'rb')
parmsin,fieldsin=pickle.load(inf)
print(parmsin)
bb=fieldsin['b']
print(np.max(bb),np.min(bb))
